#!/usr/bin/env node

'use strict';

var child_process = require('child_process'),
    https = require('https');

function fetchPackage(packageName, callback) {
  https.get('https://registry.npmjs.org/' + packageName, function(resp) {
    if (resp.statusCode == 404) {
      callback(new Error('Package not found.'), resp);
    } else if (resp.statusCode >= 300) {
      callback(new Error('Request error.'), resp);
    } else {
      var body = '';
      resp.on('data', function(data) {
        body += data.toString();
      });
      resp.on('end', function() {
        callback(null, body);
      });
    }
  }).on('error', function(e) {
    callback(e);
  });
}

function label(name, version) {
  return version ? (name + '@' + version) : name;
}

function error(name, version, message) {
  console.error('error: ' + label(name, version) + ': ' + message);
}

function main() {
  var args = process.argv.slice(2);

  if (args.indexOf('-h') !== -1 || args.indexOf('--help') !== -1) {
    console.error('usage: readme-remote <package>[@version] [<package[@version]...]\n' +
                  '\n' +
                  'Example:\n' +
                  '$ readme-remote react mobx@5.5.0');
    process.exit(2);
  }

  args.forEach(function(arg) {
    var version;
    if (arg.indexOf('@') !== -1) {
      version = arg.split('@')[1];
      arg = arg.split('@')[0];
    }

    fetchPackage(arg, function(e, body) {
      if (e) {
        error(arg, version, e.message);
        return;
      }

      try {
        var o = JSON.parse(body);
      } catch (e) {
        error(arg, version, e.message);
        return;
      }

      var latestVersion = o['dist-tags'] && o['dist-tags'].latest;

      // Version provided.
      if (version) {
        var versionO = o.versions && o.versions[version];
        if (!versionO) {
          error(arg, version, 'Version not found.');
          return;
        }
        // Use top-level README if .latest is same as specified.
        if (latestVersion !== version || !o.readme) {
          o = versionO;
        }
      } else {
        version = latestVersion;
        var latest = o.versions[version];
        // Use latest if it has a README
        if (latest && latest.readme) {
          o = latest;
        } else {
          // No README for latest, search forward for a version with a README.
          // Not sure why some versions lack a README.
          var versions = Object.keys(o.versions).sort();
          if (versions.length) {
            var nextVersions = versions.slice(versions.indexOf(latestVersion) + 1);
            while (nextVersions.shift()) {
              var next = o.versions[nextVersions[0]];
              if (next && next.readme) {
                version = nextVersions[0];
                o = next;
                break;
              }
            }
          }
        }
      }

      if (o) {
        var width = process.stdout.columns || process.env.COLUMNS;
        if (!width) {
          // Needed for pttys like `less`.
          var tputCols = child_process.spawnSync('tput', ['cols']);
          if (!tputCols.error) {
            width = parseInt(tputCols.stdout.toString());
          }
        }

        console.log('README: ' + arg + '@' + version + '\n');
        if (o.readme) {
          console.log(o.readme);
        } else {
          console.log('No README for this version.');
        }
        console.log(new Array(width || 80).join('-') + '\n');
      }
    });
  });
}

main();
